@@ -1,3 +1,5 @@
+#' @include url.R
+
 # Build a request for the REST protocol.
 rest_build <- function(request) {
   if (params_filled(request)) {
@@ -227,10 +229,10 @@ clean_path <- function(url) {
 
 # Return a string with special characters escaped, e.g. " " -> "%20".
 escape_path <- function(string, encode_sep) {
-  path <- utils::URLencode(string, TRUE)
-  if (!encode_sep) {
-    path <- gsub("%2F", "/", path)
+  if(!encode_sep){
+    base_url_encode <- paste0("/", base_url_encode)
   }
+  path <- paws_url_encoder(string, paste0("[^", base_url_encode, "]"))
   return(path)
 }
 
